using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Runtime.Serialization;

using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Grouping;
using Syncfusion.Collections.BinaryTree;


namespace GGC_FilterBar_DropDownUserControl
{
	public class CustomFilterBarCellModel : GridTableFilterBarCellModel
	{		
		internal DropDownFormAndUserControlSample.DropDownUser dropdown;

		public CustomFilterBarCellModel(GridModel grid)	: base(grid)
		{			
			dropdown = new DropDownFormAndUserControlSample.DropDownUser();
		}

		public override GridCellRendererBase CreateRenderer(GridControlBase control)
		{
			DropDownUserCellModel cellmodel = new DropDownUserCellModel(this.Grid,new DropDownFormAndUserControlSample.DropDownUser());
			return new DropDownUserCellRenderer(control, cellmodel);			
		}				
	}

	public class DropDownUserCellModel : GridStaticCellModel
	{
		internal DropDownFormAndUserControlSample.DropDownUser ddUser;
		internal static string selectCustomText;

		protected DropDownUserCellModel(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
			ButtonBarSize = new Size(14, 14);
		}

		public DropDownUserCellModel(GridModel grid, DropDownFormAndUserControlSample.DropDownUser ddUser)
			: base(grid)
		{
			this.ddUser = ddUser;
			ButtonBarSize = new Size(14, 14);
		}
	
		public override GridCellRendererBase CreateRenderer(GridControlBase control)
		{
			return new DropDownUserCellRenderer(control, this);
		}		

		public string SelectCustomText
		{
			get
			{
				return selectCustomText;
			}
			set
			{
				selectCustomText = value;
			}
		}

		public void Select(GridTableCellStyleInfoIdentity tableCellIdentity, string s)
		{
			if (s.Length >= 0)
			{
				if (s == "Reset")
				{
					ResetFilterBar(tableCellIdentity);
				}		
				else
				{
					SelectItem(tableCellIdentity, s);
				}
			}
		}

		protected void SelectItem(GridTableCellStyleInfoIdentity tableCellIdentity, string s)
		{
			RecordFilterDescriptor filter = tableCellIdentity.Table.TableDescriptor.RecordFilters[tableCellIdentity.Column.Name];
			object[] items = GetFilterBarChoices(tableCellIdentity);
			RecordFilterDescriptor newFilter = new RecordFilterDescriptor();
			newFilter.Name = tableCellIdentity.Column.Name;
			string[] s1 = s.Split(',');
			foreach (string b in s1)
			{	
				newFilter.Conditions.Add(new FilterCondition(FilterCompareOperator.Equals, b));
			}	
			
			if (filter == null)
				tableCellIdentity.Table.TableDescriptor.RecordFilters.Add(newFilter);
			else
				filter.InitializeFrom(newFilter);
		}

		 
		public void FillWithChoices(GridStyleInfo style)
		{			
			GridTableCellStyleInfo tableStyleInfo = (GridTableCellStyleInfo) style;
			object[] items = (object[]) GetFilterBarChoices(tableStyleInfo.TableCellIdentity);

			CheckedListBox listBox = this.ddUser.checkedListBox1;
			listBox.Items.Clear();

			if (items != null)
			{	
				listBox.Items.Add("Reset");	
				
				foreach (object item in items)
				{									
					if(!(item.ToString() == null || item.ToString() == "") && item != null)						
						listBox.Items.Add(style.GetFormattedText(item));					
				}				
			}			
		}


		public object[] GetFilterBarChoices(GridTableCellStyleInfoIdentity tableCellIdentity)
		{
			object[] dataSource = null;

			if (tableCellIdentity.FilterBarSummaryDescriptor != null)
			{
				ITreeTableSummary[] summaries = tableCellIdentity.Table.GetSummaries();
				int summaryIndex = tableCellIdentity.Table.TableDescriptor.Summaries.IndexOf(tableCellIdentity.FilterBarSummaryDescriptor);
				FilterBarChoicesSummary filterBarChoicesSummary = (FilterBarChoicesSummary) summaries[summaryIndex];
				return filterBarChoicesSummary.Values;
			}
			return dataSource;
		}

		protected void ResetFilterBar(GridTableCellStyleInfoIdentity tableCellIdentity)
		{
			RecordFilterDescriptor filter = tableCellIdentity.Table.TableDescriptor.RecordFilters[tableCellIdentity.Column.Name];
			if (filter != null)
			{
				tableCellIdentity.Table.TableDescriptor.RecordFilters.Remove(tableCellIdentity.Column.Name);
			}
		}

		protected DialogResult ShowCollectionDialog(object instance, string propertyName, IServiceProvider provider, Type type)
		{			
			GroupingCollectionEditor ce = new GroupingCollectionEditor(type);
			Syncfusion.ComponentModel.WindowsFormsEditorServiceContainer esc = new Syncfusion.ComponentModel.WindowsFormsEditorServiceContainer(provider);
			PropertyDescriptor pd = TypeDescriptor.GetProperties(instance)[propertyName];
			Syncfusion.ComponentModel.TypeDescriptorContext tdc = new Syncfusion.ComponentModel.TypeDescriptorContext(instance, pd);
			tdc.ServiceProvider = esc;

			ce.EditValue(tdc, esc, pd.GetValue(instance));

			return esc.DialogResult;
		}		
	}

	public class DropDownUserCellRenderer : GridStaticCellRenderer
	{
		internal DropDownFormAndUserControlSample.DropDownUser ddUser;
		internal GridTableCellStyleInfoIdentity tableCellIdentity;		
		internal GridTableControl grid;
		internal string[] cellValue = null;	

		public DropDownUserCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
			: base(grid, cellModel)
		{
			this.ddUser = ((DropDownUserCellModel)this.Model).ddUser;
			this.ddUser.Visible = false;
			DropDownPart = new GridDropDownCellImp(this);
			DropDownButton = new GridCellComboBoxButton(this);
			this.grid = grid as GridTableControl;		
			Console.WriteLine(grid.Model.ColCount);
			cellValue = new string[grid.Model.ColCount];
			
			this.ddUser.UserControlSave += new EventHandler(user_Save);
			this.ddUser.UserControlCancel += new EventHandler(user_Cancel);				
		}

		# region DropDownContrainer....
		public new GridDropDownContainer DropDownContainer
		{
			get{return (GridDropDownContainer)base.DropDownContainer;}
		}
			
		protected  override void InitializeDropDownContainer()
		{
			base.InitializeDropDownContainer();
			if (this.DropDownContainer != null)
			{
				this.DropDownContainer.Controls.Add(this.ddUser);
			}
		}
		# endregion

		public override void DropDownContainerShowingDropDown(object sender, CancelEventArgs e)
		{				
			this.Model.FillWithChoices(StyleInfo);
			CheckedListBox listBoxPart = (CheckedListBox) ddUser.checkedListBox1;					
			
			if(Grid.GetColWidth(ColIndex) < 80 )  //80 - Minimal width
			{
				this.DropDownContainer.Width = 80;
				this.ddUser.Width = 80;
			}
			else
			{
				this.DropDownContainer.Width = Grid.GetColWidth(ColIndex);				
				this.ddUser.Width = Grid.GetColWidth(ColIndex);
			}

			this.DropDownContainer.Height = this.ddUser.Height;
			ddUser.Visible = true;
			this.ddUser.SetValuesFromString(cellValue[this.grid.CurrentCell.ColIndex]);
		}
		
		public override void DropDownContainerCloseDropDown(object sender, PopupClosedEventArgs e)  
		{
			if (e.PopupCloseType == PopupCloseType.Done)
			{
				if (this.NotifyCurrentCellChanging())
				{					
					this.NotifyCurrentCellChanged();		
					Model.Select(tableCellIdentity,this.ddUser.GetValuesToString());
					cellValue[this.grid.CurrentCell.ColIndex] = GetFilterBarText(StyleInfo);
					Console.WriteLine("DropDownContainerCloseDropDown" + cellValue[this.grid.CurrentCell.ColIndex]);
				}
			}						
			base.DropDownContainerCloseDropDown(sender, e);
		}

		private void user_Save(object sender, EventArgs e)
		{			
			this.grid.CurrentCell.CloseDropDown(Syncfusion.Windows.Forms.PopupCloseType.Done);			
		}

		public override void OnPrepareViewStyleInfo(GridPrepareViewStyleInfoEventArgs e)
		{
			if(this.grid.CurrentCell.ColIndex == e.ColIndex)
				e.Style.CellValue = GetFilterBarText(e.Style);			
			base.OnPrepareViewStyleInfo(e);
		}
	
		private void user_Cancel(object sender, EventArgs e)
		{			
			this.grid.CurrentCell.CloseDropDown(Syncfusion.Windows.Forms.PopupCloseType.Canceled);			
		}
		
		protected override void OnInitialize(int rowIndex, int colIndex)
		{					
			GridTableCellStyleInfo tableStyleInfo = (GridTableCellStyleInfo) StyleInfo;
			tableCellIdentity = tableStyleInfo.TableCellIdentity;				
			base.OnInitialize (rowIndex, colIndex);
		}		

		public new DropDownUserCellModel Model
		{
			get
			{
				return (DropDownUserCellModel) base.Model;
			}
		}

		protected override void OnDraw(Graphics g, Rectangle clientRectangle, int rowIndex, int colIndex, GridStyleInfo style)
		{			
			style.CellValue = GetFilterBarText(style);
			base.OnDraw (g, clientRectangle, rowIndex, colIndex, style);
		}

		public string GetFilterBarText(GridStyleInfo style)
		{
			GridTableCellStyleInfo tableStyleInfo = (GridTableCellStyleInfo) style;
			GridTableCellStyleInfoIdentity tableCellIdentity = tableStyleInfo.TableCellIdentity;
			RecordFilterDescriptor filter = tableCellIdentity.Table.TableDescriptor.RecordFilters[tableCellIdentity.Column.Name];
		
			object value = Model.SelectCustomText;
			if (filter == null)
				value = "";
			else if (filter.Conditions.Count >0)
			{				
				for(int i=0; i< filter.Conditions.Count; i++)				
					value += filter.Conditions[i].CompareValue + ",";
			}
			return value.ToString();
		}

		public object[] GetFilterBarChoices(GridTableCellStyleInfoIdentity tableCellIdentity)
		{
			object[] dataSource = null;

			if (tableCellIdentity.FilterBarSummaryDescriptor != null)
			{
				ITreeTableSummary[] summaries = tableCellIdentity.Table.GetSummaries();
				int summaryIndex = tableCellIdentity.Table.TableDescriptor.Summaries.IndexOf(tableCellIdentity.FilterBarSummaryDescriptor);
				FilterBarChoicesSummary filterBarChoicesSummary = (FilterBarChoicesSummary) summaries[summaryIndex];
				return filterBarChoicesSummary.Values;
			}

			return dataSource;
		}

	}
}
